<?php
/**
 * SM FAQ - simple faq component for Joomla! 1.5
 * @version 1.3.2
 * @package SM FAQ
 * @author SmokerMan
 * @author url http://joomla-code.ru
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
 */

// No direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport('joomla.application.component.model');

class SmfaqModelSmfaq extends JModel
{
	/**
	 * SMFaq id
	 *
	 * @var int
	 */
	var $_id = null;

	/**
	 * SMFaq data
	 *
	 * @var array
	 */
	var $_data = null;
	var $_comments = null;

	/**
	 * Constructor
	 *
	 * @since 1.5
	 */
	function __construct()
	{
		parent::__construct();

		$array = JRequest::getVar('cid',  0, '', 'array');
		$edit	= JRequest::getVar('edit',true);
		if($edit)
			$this->setId((int)$array[0]);
	}

	function setId($id)
	{
		// Set id and wipe data
		$this->_id		= $id;
		$this->_data	= null;
		$this->_comments = null;
	}

	function &getData()
	{
		// Load the data
		if ($this->_loadData()) {
			$user = &JFactory::getUser();

			// Check to see if the category is published
			if (!$this->_data->cat_pub) {
				JError::raiseError( 404, JText::_("Resource Not Found") );
				return;
			}

			// Check whether category access level allows access
			if ($this->_data->cat_access > $user->get('aid', 0)) {
				JError::raiseError( 403, JText::_('ALERTNOTAUTH') );
				return;
			}
		}
		else $this->_initData();
		return $this->_data;
	}

	function isCheckedOut( $uid=0 )
	{
		if ($this->_loadData())
		{
			if ($uid) {
				return ($this->_data->checked_out && $this->_data->checked_out != $uid);
			} else {
				return $this->_data->checked_out;
			}
		}
	}

	/**
	 * Method to checkin/unlock the weblink
	 *
	 * @access	public
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function checkin()
	{
		if ($this->_id)
		{
			$smfaq = & $this->getTable();
			if(! $smfaq->checkin($this->_id)) {
				$this->setError($this->_db->getErrorMsg());
				return false;
			}
		}
		return false;
	}

	/**
	 * Method to checkout/lock the weblink
	 *
	 * @access	public
	 * @param	int	$uid	User ID of the user checking the article out
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function checkout($uid = null)
	{
		if ($this->_id)
		{
			// Make sure we have a user id to checkout the article with
			if (is_null($uid)) {
				$user	=& JFactory::getUser();
				$uid	= $user->get('id');
			}
			// Lets get to it and checkout the thing...
			$smfaq = & $this->getTable();
			if(!$smfaq->checkout($uid, $this->_id)) {
				$this->setError($this->_db->getErrorMsg());
				return false;
			}

			return true;
		}
		return false;
	}

	function _loadData()
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_data))
		{
			$query = 'SELECT a.*, cc.title AS category, cc.published AS cat_pub, cc.access AS cat_access, v.vote_yes AS vote_yes, v.vote_no AS vote_no'.
					' FROM #__smfaq AS a' .
					' LEFT JOIN #__categories AS cc ON cc.id = a.catid' .
					' LEFT JOIN #__smfaq_votes AS v ON a.id = v.question_id ' .
					' WHERE a.id = '.(int) $this->_id;
			$this->_db->setQuery($query);
			$this->_data = $this->_db->loadObject();
			return (boolean) $this->_data;
		}
		return true;
	} 
	
	function &getComments()
	{
		if (empty($this->_comments))
		{
			$query = 'SELECT a.*'.
					' FROM #__smfaq_comments AS a' .
					' WHERE a.question_id = '.(int) $this->_id;
			$this->_db->setQuery($query);
			$this->_comments = $this->_db->loadObjectList();
			return $this->_comments;
		}
	}
	
	function _initData()
	{
		// Lets load the content if it doesn't already exist
		
			$smfaq = new stdClass();
			$smfaq->id 				= 0;
			$smfaq->catid		    = 0;
			$smfaq->question		= null;
			$smfaq->answer			= null;
			$smfaq->state			= 0;
			$smfaq->created			= null;
			$smfaq->email			= null;
			$smfaq->published    	= 0;
			$smfaq->ordering    	= 0;
			$smfaq->created_by		= null;
			$smfaq->created			= null;
			$smfaq->created_ans		= null;
			$smfaq->send_mail 		= 0;
			$smfaq->ip		 		= null;
			$smfaq->userid			= 0;
			$smfaq->vote_yes		= 0;
			$smfaq->vote_no			= 0;
			$smfaq->checked_out			= 0;
			$smfaq->checked_out_time	= 0;
			
			$this->_data			= $smfaq;
			return (boolean) $this->_data;
		
		return true;
	}
	
	function store($data)
	{	
		$row =& $this->getTable();
		$user	=& JFactory::getUser();
		$params = JRequest::getVar ('params', array(), 'post', 'array' );
		
		if ($user->gid > '23') {
			$data['answer'] = JRequest::getVar('answer', '', 'post', 'string', JREQUEST_ALLOWRAW);
		} else {
			$data['question'] = JRequest::getVar('question', '', 'post', 'string', JREQUEST_ALLOWHTML);
		}
		
		$config =& JFactory::getConfig();
		$tzoffset = $config->getValue('config.offset');
		$date =& JFactory::getDate($data['created'], $tzoffset);
		$date_ans =& JFactory::getDate($data['created_ans'], $tzoffset);
		$data['created'] = $date->toMySQL();
		$data['created_ans'] = $date_ans->toMySQL();

		
		$row->bind($params);
		
		if (!$row->bind($data)) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		if (!$row->check()) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		if (!$row->store()) {
			$this->setError( $row->getErrorMsg() );
			return false;
		}
		JRequest::setVar('id',$row->id); 

		return true;
	}

	/**
	 * Method to delete record(s)
	 *
	 * @return	boolean	True on success
	 */
	function delete($cid = array())
	{
		$result = false;

		if (count( $cid ))
		{
			JArrayHelper::toInteger($cid);
			$cids = implode( ',', $cid );
			$query = 'DELETE FROM #__smfaq'
				. ' WHERE id IN ( '.$cids.' )';
			$this->_db->setQuery( $query );
			if(!$this->_db->query()) {
				$this->setError($this->_db->getErrorMsg());
				return false;
			}
			$query = 'DELETE FROM #__smfaq_votes'
				. ' WHERE question_id IN ( '.$cids.' )';
			$this->_db->setQuery( $query );
			if(!$this->_db->query()) {
				$this->setError($this->_db->getErrorMsg());
				return false;
			}
		}

		return true;
	}
	
	/**
	 * Method to (un)publish
	 *
	 * @return	boolean	True on success
	 */
	function publish($cid = array(), $publish = 1)
	{
		$user 	=& JFactory::getUser();

		if (count( $cid ))
		{
			JArrayHelper::toInteger($cid);
			$cids = implode( ',', $cid );

			$query = 'UPDATE #__smfaq'
				. ' SET published = '.(int) $publish
				. ' WHERE id IN ( '.$cids.' )'
				. ' AND ( checked_out = 0 OR ( checked_out = '.(int) $user->get('id').' ) )'
			;
			$this->_db->setQuery( $query );
			if (!$this->_db->query()) {
				$this->setError($this->_db->getErrorMsg());
				return false;
			}
		}

		return true;
	}
	
	
	/**
	 * Method to move
	 *
	 * @access	public
	 * @return	boolean	True on success
	 */
	function move($direction)
	{
		$row =& $this->getTable();
		if (!$row->load($this->_id)) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		if (!$row->move( $direction, ' catid = '.(int) $row->catid.' AND published >= 0 ' )) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		return true;
	}

	/**
	 * Method to move a saveorder
	 *
	 * @access	public
	 * @return	boolean	True on success
	 */
	function saveorder($cid = array(), $order)
	{
		$row =& $this->getTable();
		$groupings = array();

		// update ordering values
		for( $i=0; $i < count($cid); $i++ )
		{
			$row->load( (int) $cid[$i] );
			// track categories
			$groupings[] = $row->catid;

			if ($row->ordering != $order[$i])
			{
				$row->ordering = $order[$i];
				if (!$row->store()) {
					$this->setError($this->_db->getErrorMsg());
					return false;
				}
			}
		}

		// execute updateOrder for each parent group
		$groupings = array_unique( $groupings );
		foreach ($groupings as $group){
			$row->reorder('catid = '.(int) $group);
		}

		return true;
	}

}