<?php
/**
 * SM FAQ - simple faq component for Joomla! 1.5
 * @version 1.3.2
 * @package SM FAQ
 * @author SmokerMan
 * @author url http://joomla-code.ru
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
 */

// No direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport('joomla.application.component.model');

class SmfaqModelSmfaq extends JModel
{
	/**
	 * SMFaq id
	 *
	 * @var int
	 */
	var $_id = null;

	/**
	 * SMFaq data
	 *
	 * @var array
	 */
	var $_data = null;

	/**
	 * Constructor
	 *
	 * @since 1.5
	 */
	function __construct()
	{
		parent::__construct();

		$id = JRequest::getVar('id', 0, '', 'int');
		$this->setId((int)$id);
		
	}

	function setId($id)
	{
		// Set weblink id and wipe data
		$this->_id		= $id;
		$this->_data	= null;
	}


	function &getData()
	{
		// Load the data
		if ($this->_loadData())
		{
			// access
			$user = &JFactory::getUser();
			if ($user->get('aid', 0) < 2) {
				JError::raiseError( 403, JText::_('ALERTNOTAUTH') );
				return;
			}

			// Check to see if the category is published
			if (!$this->_data->cat_pub) {
				JError::raiseError( 404, JText::_("Resource Not Found") );
				return;
			}

			// Check whether category access level allows access
			if ($this->_data->cat_access > $user->get('aid', 0)) {
				JError::raiseError( 403, JText::_('ALERTNOTAUTH') );
				return;
			}
		}
		else  $this->_initData();
		
		return $this->_data;
	}
	
	function isCheckedOut( $uid=0 )
	{
		if ($this->_id < 1) {
			return false;
		}
		if ($this->_loadData())
		{
			if ($uid) {
				return ($this->_data->checked_out && $this->_article->checked_out != $uid);
			} else {
				return $this->_data->checked_out;
			}
		} 
	}


	/**
	 * Method to checkin/unlock 
	 *
	 * @access	public
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function checkin()
	{
		if ($this->_id)
		{
			$smfaq = & $this->getTable();
			if(! $smfaq->checkin($this->_id)) {
				$this->setError($this->_db->getErrorMsg());
				return false;
			}
		}
		return false;
	}

	/**
	 * Method to checkout/lock the 
	 *
	 * @access	public
	 * @param	int	$uid	User ID of the user checking the article out
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function checkout($uid = null)
	{
		if ($this->_id)
		{
			// Make sure we have a user id to checkout the article with
			if (is_null($uid)) {
				$user	=& JFactory::getUser();
				$uid	= $user->get('id');
			}
			// Lets get to it and checkout the thing...
			$smfaq = & $this->getTable();
			if(!$smfaq->checkout($uid, $this->_id)) {
				$this->setError($this->_db->getErrorMsg());
				return false;
			}

			return true;
		}
		return false;
	}
	
	function store($post)
	{
		$datenow =& JFactory::getDate();
		$user = & JFactory::getUser();
		$post['id'] = JRequest::getInt('id');
		if ($user->get('gid') < 19 ) {
			$post['id'] = null;
			unset($post['created_ans'], $post['answer'], $post['state'], 
					$post['created_ans_by'], $post['published'], $post['params']);
		} 
		
		$row =& $this->getTable();
		
		
		if ($post['id'] < 1) {
			$post['created'] = $datenow->toMySQL();
		} else {
			$config =& JFactory::getConfig();
			$tzoffset = $config->getValue('config.offset');
			$date =& JFactory::getDate($post['created'], $tzoffset);
			$date_ans = & JFactory::getDate($post['created_ans'], $tzoffset);
			$post['created'] = $date->toMySQL();
			$post['created_ans'] = $date_ans->toMySQL();
		}
		// Bind the form fields to  table
		if (!$row->bind($post)) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		// if new item, order last in appropriate group
		if (!$row->id) {
			$where = 'catid = ' . (int) $row->catid ;
			$row->ordering = $row->getNextOrder( $where );
		}
		// Make sure table is valid
		if (!$row->check()) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		// Store table to the database
		if (!$row->store()) {
			$this->setError($this->_db->getErrorMsg());
			return false;
		}

		return true;
	}
	
	function _loadData()
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_data))
		{
			$query = 'SELECT a.*, cc.title AS category,' .
					' cc.published AS cat_pub, cc.access AS cat_access, v.question_id AS voiting '.
					' FROM #__smfaq AS a' .
					' LEFT JOIN #__categories AS cc ON cc.id = a.catid' .
					' LEFT JOIN #__smfaq_votes AS v ON v.question_id = a.id ' .
					' WHERE a.id = '. (int) $this->_id;
			$this->_db->setQuery($query);
			$this->_data = $this->_db->loadObject();			
			return (boolean) $this->_data;
		}
		return true;
	}
	
	function _initData()
	{
		
		$user = &JFactory::getUser();
		$catid = JRequest::getInt('catid');
		
		$query = 'SELECT c.id, c.access, c.published ' .
		' FROM #__categories AS c' .
		' WHERE c.id = '. (int) $catid .
		' AND c.section = "com_smfaq"';
		$this->_db->setQuery($query);
		$category = $this->_db->loadObject();
		
		if (($category->id < 1) || !$category->published ) {
			JError::raiseError( 404, JText::_("Resource Not Found") );
			return;
		}
		if ($category->access > $user->get('aid', 0)) {
			JError::raiseError( 403, JText::_('ALERTNOTAUTH') );
			return;
		}
		if (empty($this->_data))
		{
			$smfaq = new stdClass();
			$smfaq->id 				= 0;
			$smfaq->catid 			= 0;
			$smfaq->question		= null;
			$smfaq->email			= null;
			$smfaq->created_by		= null;
			$smfaq->send_mail		= null;
			if ($user->get('gid') > 19) {
				$smfaq->published  		= 0;
				$smfaq->state			= 0;
				$smfaq->answer			= null;
				$smfaq->created 		= null;
				$smfaq->ip				= null;
				$smfaq->voiting			= null;
			}
			
			$this->_data			= $smfaq;
			return (boolean) $this->_data;
		}
		return true;
	}
	
	function getComments()
	{
		$query = 'SELECT comment, id' .
				' FROM #__smfaq_comments' .
				' WHERE question_id = '. (int) $this->_id;
		$this->_db->setQuery($query);
		$comments = $this->_db->loadObjectList();			
			
		return $comments;
	}

}